@@ -8,7 +8,7 @@ from doit import logger
 
 
 class InvalidTask(Exception):
-    """Invalid task instance. It doesnt if try to execute."""
+    """Invalid task instance. User error on specifying the task."""
     pass
 
 class TaskFailed(Exception):
@@ -27,31 +27,65 @@ class BaseTask(object):
     @cvar CAPTURE_ERR: (bool) stderr from the task to be captured
     @ivar name string 
     @ivar action see derived classes
-    @ivar dependencies list of absolute? file paths.
-    @ivar targets list of absolute? file paths.
+    @ivar dependencies list of all dependencies
+    @ivar targets list of targets
+    @ivar folder_dep: (list) 
+    @ivar task_dep: (list)
+    @ivar file_dep: (list)
+    @ivar run_once: (bool) task without dependencies should run
     """
 
     CAPTURE_OUT = False
     CAPTURE_ERR = False
 
     def __init__(self,name,action,dependencies=(),targets=()):
-        """Init."""
+        """Init."""    
         # dependencies parameter must be a list
-        if not(isinstance(dependencies,list) or isinstance(dependencies,tuple)):
-            raise InvalidTask("'dependencies' paramater must be a list or" +
-                              "tuple got:'%s' => %s"% 
-                              (str(dependencies),dependencies.__class__))
+        if not ((isinstance(dependencies,list)) or
+                (isinstance(dependencies,tuple))):
+            msg = ("%s. paramater 'dependencies' must be a list or " +
+                   "tuple got:'%s'%s")
+            raise InvalidTask(msg%(name, str(dependencies),type(dependencies)))
 
         # targets parameter must be a list
         if not(isinstance(targets,list) or isinstance(targets,tuple)):
-            raise InvalidTask("'targets' paramater must be a list or tuple " +
-                             "got:'%s' => %s"% (str(targets),targets.__class__))
+            msg = ("%s. paramater 'targets' must be a list or tuple " +
+                   "got:'%s'%s")
+            raise InvalidTask(msg % (name, str(targets),type(targets)))
 
         self.name = name
         self.action = action
         self.dependencies = dependencies
         self.targets = targets
+        self.run_once = False        
 
+        # there are 3 kinds of dependencies: file, task, and folder
+        self.folder_dep = [] 
+        self.task_dep = [] 
+        self.file_dep = []
+        for dep in self.dependencies:
+            # True on the list. set run_once
+            if isinstance(dep,bool):
+                if not dep:
+                    msg = ("%s. bool paramater in 'dependencies' "+
+                           "must be True got:'%s'")
+                    raise InvalidTask(msg%(name, str(dep)))
+                self.run_once = True
+            # folder dep ends with a '/'
+            elif dep.endswith('/'):
+                self.folder_dep.append(dep)
+            # task dep starts with a ':'
+            elif dep.startswith(':'):                    
+                self.task_dep.append(dep[1:])
+            # file dep 
+            elif isinstance(dep,str):
+                self.file_dep.append(dep)
+
+        # run_once can't be used together with file dependencies
+        if self.run_once and self.file_dep:
+            msg = ("%s. task cant have file and dependencies and True " +
+                   "at the same time. (just remove True)")
+            raise InvalidTask(msg % name)
 
     def execute(self):
         """Executes the task.
@@ -62,10 +96,6 @@ class BaseTask(object):
         raise InvalidTask("Not Implemented")
 
 
-    def __str__(self):
-        return str(self.action)
-
-
     def title(self):
         """String representation on output.
 
@@ -74,7 +104,6 @@ class BaseTask(object):
         return "%s => %s"% (self.name,str(self))
 
 
-
 class CmdTask(BaseTask):
     """Command line task. Spawns a new process."""
  
@@ -87,11 +116,11 @@ class CmdTask(BaseTask):
     def execute(self):
         # set Popen stream parameters
         if not self.CAPTURE_OUT:
-            stdout = sys.__stdout__
+            stdout = None
         else:
             stdout = subprocess.PIPE
         if not self.CAPTURE_ERR:
-            stderr = sys.__stderr__
+            stderr = None
         else:
             stderr = subprocess.PIPE
 
@@ -110,11 +139,13 @@ class CmdTask(BaseTask):
         # http://www.gnu.org/software/bash/manual/bashref.html#Exit-Status
         # it doesnt make so much difference to return as Error or Failed anyway
         if process.returncode > 125:
-            raise TaskError("")
+            raise TaskError("Command error: '%s' returned %s" % 
+                            (self.action,process.returncode))
 
         # task failure
         if process.returncode != 0:
-            raise TaskFailed("")
+            raise TaskFailed("Command failed: '%s' returned %s" % 
+                             (self.action,process.returncode))
 
             
     def __str__(self):
@@ -159,12 +190,14 @@ class PythonTask(BaseTask):
 
         # execute action / callable
         try:
-            result = self.action(*self.args,**self.kwargs)
-        except Exception, exception:
-            error = TaskError(exception)
-            error.originalException = traceback.format_exception(\
-                exception.__class__, exception,sys.exc_info()[2])
-            raise error
+            # Python2.4
+            try:
+                result = self.action(*self.args,**self.kwargs)
+            except Exception, exception:
+                error = TaskError(exception)
+                error.originalException = traceback.format_exception(\
+                    exception.__class__, exception,sys.exc_info()[2])
+                raise error
         finally:
             # restore std streams /log captured streams
             if self.CAPTURE_OUT:
@@ -179,7 +212,8 @@ class PythonTask(BaseTask):
 
         # if callable returns false. Task failed
         if not result:
-            raise TaskFailed("")
+            raise TaskFailed("Python Task failed: '%s' returned %s" %
+                             (self.action, result))
                     
     def __str__(self):
         # get object description excluding runtime memory address
@@ -187,3 +221,18 @@ class PythonTask(BaseTask):
 
     def __repr__(self):
         return "<PythonTask: %s - '%s'>"% (self.name,repr(self.action))
+
+
+
+class GroupTask(BaseTask):
+    """Do nothing. Used to create group tasks
+    Group is actually defined by dependencies.
+    """
+    def execute(self):
+        pass
+
+    def __str__(self):
+        return "Group: %s" % ", ".join(self.dependencies)
+
+    def __repr__(self):
+        return "<GroupTask: %s>"% self.name
